==> create-fly-1.21.10-6.0.8-5-sources.jar: com/zurrtum/create/client/infrastructure/model/CopycatPanelModel.java <==
package com.zurrtum.create.client.infrastructure.model;

import com.zurrtum.create.AllBlocks;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.client.catnip.render.SpriteShiftEntry;
import com.zurrtum.create.client.foundation.model.BakedModelHelper;
import com.zurrtum.create.client.foundation.model.BakedQuadHelper;
import com.zurrtum.create.content.decoration.copycat.CopycatBlock;
import com.zurrtum.create.content.decoration.copycat.CopycatPanelBlock;
import com.zurrtum.create.content.decoration.copycat.CopycatSpecialCases;
import com.zurrtum.create.foundation.block.WrenchableDirectionalBlock;
import net.minecraft.class_1058;
import net.minecraft.class_10801;
import net.minecraft.class_10817;
import net.minecraft.class_1087;
import net.minecraft.class_10889;
import net.minecraft.class_1920;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_238;
import net.minecraft.class_243;
import net.minecraft.class_2680;
import net.minecraft.class_5819;
import net.minecraft.class_777;
import net.minecraft.client.render.model.*;
import java.util.List;
import java.util.function.Consumer;

public class CopycatPanelModel extends CopycatModel {
    protected static final class_238 CUBE_AABB = new class_238(class_2338.field_10980);

    public CopycatPanelModel(class_2680 state, class_9979 unbaked) {
        super(state, unbaked);
    }

    @Override
    public void addPartsWithInfo(
        class_1920 world,
        class_2338 pos,
        class_2680 state,
        CopycatBlock block,
        class_2680 material,
        class_5819 random,
        List<class_10889> parts
    ) {
        if (CopycatSpecialCases.isTrapdoorMaterial(material)) {
            addModelParts(world, pos, material, random, getModelOf(material), parts);
            return;
        }
        OcclusionData occlusionData = gatherOcclusionData(world, pos, state, material, block);
        if (CopycatSpecialCases.isBarsMaterial(material)) {
            class_2350 facing = state.method_61767(CopycatPanelBlock.FACING, class_2350.field_11036);
            class_2680 bars = AllBlocks.COPYCAT_BARS.method_9564().method_11657(WrenchableDirectionalBlock.field_10927, facing);
            class_1087 model = getModelOf(material);
            addBarsParts(
                occlusionData,
                state,
                block,
                model.method_68511(),
                getMaterialParts(world, pos, material, random, model),
                getMaterialParts(world, pos, material, random, getModelOf(bars)),
                parts
            );
        } else {
            addPanelParts(occlusionData, state, block, getMaterialParts(world, pos, material, random, getModelOf(material)), parts);
        }
    }

    protected void addBarsParts(
        OcclusionData occlusionData,
        class_2680 state,
        CopycatBlock block,
        class_1058 particle,
        List<class_10889> material,
        List<class_10889> original,
        List<class_10889> parts
    ) {
        boolean vertical = state.method_11654(CopycatPanelBlock.FACING).method_10166() == class_2351.field_11052;
        class_1058 findSprite = null;
        for (class_10889 part : original) {
            class_10817.class_10818 builder = new class_10817.class_10818();
            addBarsCroppedQuads(particle, part.method_68509(null), builder::method_68051);
            for (class_2350 direction : Iterate.directions) {
                if (occlusionData.isOccluded(direction))
                    continue;
                List<class_777> quads = part.method_68509(direction);
                class_1058 targetSprite = particle;
                if (vertical || direction.method_10166() == class_2351.field_11052) {
                    if (findSprite != null) {
                        targetSprite = findSprite;
                    } else {
                        for (class_10889 materialPart : material) {
                            for (class_777 quad : materialPart.method_68509(null)) {
                                if (quad.comp_3723() != class_2350.field_11036)
                                    continue;
                                targetSprite = findSprite = quad.comp_3724();
                                break;
                            }
                        }
                        if (findSprite == null) {
                            findSprite = particle;
                        }
                    }
                }
                addBarsCroppedQuads(
                    targetSprite,
                    quads,
                    block.shouldFaceAlwaysRender(state, direction) ? builder::method_68051 : (class_777 quad) -> builder.method_68053(direction, quad)
                );
            }
            parts.add(new class_10801(builder.method_68050(), part.comp_3751(), part.comp_3752()));
        }
    }

    protected void addBarsCroppedQuads(class_1058 targetSprite, List<class_777> quads, Consumer<class_777> consumer) {
        if (targetSprite == null) {
            quads.forEach(consumer);
            return;
        }
        for (class_777 quad : quads) {
            class_1058 original = quad.comp_3724();
            class_777 newQuad = BakedQuadHelper.clone(quad);
            int[] vertexData = newQuad.comp_3721();
            for (int vertex = 0; vertex < 4; vertex++) {
                BakedQuadHelper.setU(
                    vertexData,
                    vertex,
                    targetSprite.method_4580(SpriteShiftEntry.getUnInterpolatedU(original, BakedQuadHelper.getU(vertexData, vertex)))
                );
                BakedQuadHelper.setV(
                    vertexData,
                    vertex,
                    targetSprite.method_4570(SpriteShiftEntry.getUnInterpolatedV(original, BakedQuadHelper.getV(vertexData, vertex)))
                );
            }
            consumer.accept(newQuad);
        }
    }

    protected void addPanelParts(
        OcclusionData occlusionData,
        class_2680 state,
        CopycatBlock block,
        List<class_10889> original,
        List<class_10889> parts
    ) {
        if (original.isEmpty()) {
            return;
        }
        class_2350 facing = state.method_61767(CopycatPanelBlock.FACING, class_2350.field_11036);
        class_243 normal = class_243.method_24954(facing.method_62675());
        class_243 normalScaled14 = normal.method_1021(14 / 16f);
        class_243 frontNormalScaledN13 = normal.method_1021((float) 0);
        class_243 normalScaledN13 = normal.method_1021(-13 / 16f);
        double frontContract = 15d / 16;
        double contract = 14d / 16;
        class_238 frontBB = CUBE_AABB.method_1002(normal.field_1352 * frontContract, normal.field_1351 * frontContract, normal.field_1350 * frontContract);
        class_238 bb = CUBE_AABB.method_1002(normal.field_1352 * contract, normal.field_1351 * contract, normal.field_1350 * contract).method_997(normalScaled14);
        for (class_10889 part : original) {
            class_10817.class_10818 builder = new class_10817.class_10818();
            addPanelCroppedQuads(facing, frontBB, bb, frontNormalScaledN13, normalScaledN13, part.method_68509(null), builder::method_68051);
            for (class_2350 direction : Iterate.directions) {
                if (occlusionData.isOccluded(direction))
                    continue;
                addPanelCroppedQuads(
                    facing,
                    frontBB,
                    bb,
                    frontNormalScaledN13,
                    normalScaledN13,
                    part.method_68509(direction),
                    block.shouldFaceAlwaysRender(state, direction) ? builder::method_68051 : (class_777 quad) -> builder.method_68053(direction, quad)
                );
            }
            parts.add(new class_10801(builder.method_68050(), part.comp_3751(), part.comp_3752()));
        }
    }

    protected void addPanelCroppedQuads(
        class_2350 facing,
        class_238 frontBB,
        class_238 bb,
        class_243 frontNormalScaledN13,
        class_243 normalScaledN13,
        List<class_777> quads,
        Consumer<class_777> consumer
    ) {
        int size = quads.size();
        if (size == 0) {
            return;
        }
        class_238 crop;
        class_243 move;
        for (boolean front : Iterate.trueAndFalse) {
            if (front) {
                crop = frontBB;
                move = frontNormalScaledN13;
            } else {
                crop = bb;
                move = normalScaledN13;
            }
            for (int i = 0; i < size; i++) {
                class_777 quad = quads.get(i);
                class_2350 direction = quad.comp_3723();

                if (front && direction == facing)
                    continue;
                if (!front && direction == facing.method_10153())
                    continue;

                consumer.accept(BakedQuadHelper.cloneWithCustomGeometry(
                    quad,
                    BakedModelHelper.cropAndMove(quad.comp_3721(), quad.comp_3724(), crop, move)
                ));
            }
        }
    }
}

==> create-fly-1.21.11-6.0.8-2-sources.jar: com/zurrtum/create/client/infrastructure/model/CopycatPanelModel.java <==
package com.zurrtum.create.client.infrastructure.model;

import com.zurrtum.create.AllBlocks;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.client.foundation.model.BakedModelHelper;
import com.zurrtum.create.client.model.NormalsBakedQuad;
import com.zurrtum.create.content.decoration.copycat.CopycatBlock;
import com.zurrtum.create.content.decoration.copycat.CopycatPanelBlock;
import com.zurrtum.create.content.decoration.copycat.CopycatSpecialCases;
import com.zurrtum.create.foundation.block.WrenchableDirectionalBlock;
import java.util.List;
import java.util.function.Consumer;
import net.minecraft.class_1058;
import net.minecraft.class_10801;
import net.minecraft.class_10817;
import net.minecraft.class_1087;
import net.minecraft.class_10889;
import net.minecraft.class_1920;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_238;
import net.minecraft.class_243;
import net.minecraft.class_2680;
import net.minecraft.class_5819;
import net.minecraft.class_777;

public class CopycatPanelModel extends CopycatModel {
    protected static final class_238 CUBE_AABB = new class_238(class_2338.field_10980);

    public CopycatPanelModel(class_2680 state, class_9979 unbaked) {
        super(state, unbaked);
    }

    @Override
    public void addPartsWithInfo(
        class_1920 world,
        class_2338 pos,
        class_2680 state,
        CopycatBlock block,
        class_2680 material,
        class_5819 random,
        List<class_10889> parts
    ) {
        if (CopycatSpecialCases.isTrapdoorMaterial(material)) {
            addModelParts(world, pos, material, random, getModelOf(material), parts);
            return;
        }
        OcclusionData occlusionData = gatherOcclusionData(world, pos, state, material, block);
        if (CopycatSpecialCases.isBarsMaterial(material)) {
            class_2350 facing = state.method_61767(CopycatPanelBlock.FACING, class_2350.field_11036);
            class_2680 bars = AllBlocks.COPYCAT_BARS.method_9564().method_11657(WrenchableDirectionalBlock.field_10927, facing);
            class_1087 model = getModelOf(material);
            addBarsParts(
                occlusionData,
                state,
                block,
                model.method_68511(),
                getMaterialParts(world, pos, material, random, model),
                getMaterialParts(world, pos, material, random, getModelOf(bars)),
                parts
            );
        } else {
            addPanelParts(occlusionData, state, block, getMaterialParts(world, pos, material, random, getModelOf(material)), parts);
        }
    }

    protected void addBarsParts(
        OcclusionData occlusionData,
        class_2680 state,
        CopycatBlock block,
        class_1058 particle,
        List<class_10889> material,
        List<class_10889> original,
        List<class_10889> parts
    ) {
        boolean vertical = state.method_11654(CopycatPanelBlock.FACING).method_10166() == class_2351.field_11052;
        class_1058 findSprite = null;
        for (class_10889 part : original) {
            class_10817.class_10818 builder = new class_10817.class_10818();
            addBarsCroppedQuads(particle, part.method_68509(null), builder::method_68051);
            for (class_2350 direction : Iterate.directions) {
                if (occlusionData.isOccluded(direction))
                    continue;
                List<class_777> quads = part.method_68509(direction);
                class_1058 targetSprite = particle;
                if (vertical || direction.method_10166() == class_2351.field_11052) {
                    if (findSprite != null) {
                        targetSprite = findSprite;
                    } else {
                        for (class_10889 materialPart : material) {
                            for (class_777 quad : materialPart.method_68509(null)) {
                                if (quad.comp_3723() != class_2350.field_11036)
                                    continue;
                                targetSprite = findSprite = quad.comp_3724();
                                break;
                            }
                        }
                        if (findSprite == null) {
                            findSprite = particle;
                        }
                    }
                }
                addBarsCroppedQuads(
                    targetSprite,
                    quads,
                    block.shouldFaceAlwaysRender(state, direction) ? builder::method_68051 : (class_777 quad) -> builder.method_68053(
                        direction,
                        quad
                    )
                );
            }
            parts.add(new class_10801(builder.method_68050(), part.comp_3751(), part.comp_3752()));
        }
    }

    protected void addBarsCroppedQuads(class_1058 targetSprite, List<class_777> quads, Consumer<class_777> consumer) {
        if (targetSprite == null) {
            quads.forEach(consumer);
            return;
        }
        for (class_777 quad : quads) {
            class_1058 original = quad.comp_3724();
            class_777 newQuad = new class_777(
                quad.comp_5238(),
                quad.comp_5239(),
                quad.comp_5240(),
                quad.comp_5241(),
                BakedModelHelper.calcSpriteUv(quad.comp_5242(), original, targetSprite),
                BakedModelHelper.calcSpriteUv(quad.comp_5243(), original, targetSprite),
                BakedModelHelper.calcSpriteUv(quad.comp_5244(), original, targetSprite),
                BakedModelHelper.calcSpriteUv(quad.comp_5245(), original, targetSprite),
                quad.comp_3722(),
                quad.comp_3723(),
                quad.comp_3724(),
                quad.comp_3725(),
                quad.comp_3726()
            );
            NormalsBakedQuad.setNormals(newQuad, NormalsBakedQuad.getNormals(quad));
            consumer.accept(newQuad);
        }
    }

    protected void addPanelParts(
        OcclusionData occlusionData,
        class_2680 state,
        CopycatBlock block,
        List<class_10889> original,
        List<class_10889> parts
    ) {
        if (original.isEmpty()) {
            return;
        }
        class_2350 facing = state.method_61767(CopycatPanelBlock.FACING, class_2350.field_11036);
        class_243 normal = class_243.method_24954(facing.method_62675());
        class_243 normalScaled14 = normal.method_1021(14 / 16f);
        class_243 frontNormalScaledN13 = normal.method_1021((float) 0);
        class_243 normalScaledN13 = normal.method_1021(-13 / 16f);
        double frontContract = 15d / 16;
        double contract = 14d / 16;
        class_238 frontBB = CUBE_AABB.method_1002(normal.field_1352 * frontContract, normal.field_1351 * frontContract, normal.field_1350 * frontContract);
        class_238 bb = CUBE_AABB.method_1002(normal.field_1352 * contract, normal.field_1351 * contract, normal.field_1350 * contract).method_997(normalScaled14);
        for (class_10889 part : original) {
            class_10817.class_10818 builder = new class_10817.class_10818();
            addPanelCroppedQuads(facing, frontBB, bb, frontNormalScaledN13, normalScaledN13, part.method_68509(null), builder::method_68051);
            for (class_2350 direction : Iterate.directions) {
                if (occlusionData.isOccluded(direction))
                    continue;
                addPanelCroppedQuads(
                    facing,
                    frontBB,
                    bb,
                    frontNormalScaledN13,
                    normalScaledN13,
                    part.method_68509(direction),
                    block.shouldFaceAlwaysRender(state, direction) ? builder::method_68051 : (class_777 quad) -> builder.method_68053(
                        direction,
                        quad
                    )
                );
            }
            parts.add(new class_10801(builder.method_68050(), part.comp_3751(), part.comp_3752()));
        }
    }

    protected void addPanelCroppedQuads(
        class_2350 facing,
        class_238 frontBB,
        class_238 bb,
        class_243 frontNormalScaledN13,
        class_243 normalScaledN13,
        List<class_777> quads,
        Consumer<class_777> consumer
    ) {
        int size = quads.size();
        if (size == 0) {
            return;
        }
        class_238 crop;
        class_243 move;
        for (boolean front : Iterate.trueAndFalse) {
            if (front) {
                crop = frontBB;
                move = frontNormalScaledN13;
            } else {
                crop = bb;
                move = normalScaledN13;
            }
            for (int i = 0; i < size; i++) {
                class_777 quad = quads.get(i);
                class_2350 direction = quad.comp_3723();

                if (front && direction == facing)
                    continue;
                if (!front && direction == facing.method_10153())
                    continue;

                consumer.accept(BakedModelHelper.cropAndMove(quad, crop, move));
            }
        }
    }
}
